@@ -1,9 +1,6 @@
 r"""Likelihood-free AMortized Posterior Estimation (LAMPE)"""
 
 from . import data
-from . import distributions
 from . import inference
-from . import masks
 from . import nn
-from . import plots
 from . import utils
